import type { Dictionary, Lang } from "@carefree0910/core";

import type { IToast } from "./misc";

// general

export const allMetaTypes = ["upload", "add.text", "add.sketch.path", "python.fields"] as const;
export type MetaType = (typeof allMetaTypes)[number];
export interface ICommonMetaData<T extends _IMetaData = _IMetaData> {
  alias?: string;
  timestamp?: number;
  duration?: number;
  from?: IMeta<T>;
}
type _IMetaData = ICommonMetaData & Dictionary<any>;
export interface IMeta<T extends _IMetaData = _IMetaData> {
  type: MetaType;
  data: T;
}

// specific

interface IUploadMetaData extends ICommonMetaData {
  w: number;
  h: number;
  url: string;
  isDrag: boolean;
}
export type IPythonResults = { _duration?: number } & (
  | { type: "text"; value: string[] }
  | { type: "image"; value: { w: number; h: number; url: string }[] }
);
export type IPythonFieldsMetaData = ICommonMetaData & {
  identifier: string;
  parameters: Dictionary<any>;
  response: IPythonResults;
};

export interface IMetaData {
  upload: IUploadMetaData;
  "add.text": ICommonMetaData;
  "add.sketch.path": ICommonMetaData;
  "python.fields": IPythonFieldsMetaData;
}

export interface IImportMeta<T extends MetaType> {
  t: IToast;
  lang: Lang;
  type: T;
  metaData: IMetaData[T];
}

// utils

export function checkMeta(meta: any): meta is IMeta {
  return allMetaTypes.includes(meta?.type);
}
export function getOriginMeta(meta: any): IMeta | undefined {
  if (!checkMeta(meta)) return;
  return getOriginMeta(meta.data.from);
}
export function getMetaTrace(meta: IMeta): IMeta[] {
  if (!checkMeta(meta)) return [];
  return [meta, ...(meta.data.from ? getMetaTrace(meta.data.from) : [])];
}
